#include <iostream>
#include <fstream>
#include <boost/lexical_cast.hpp>
#include <boost/tokenizer.hpp>
#include <boost/algorithm/string.hpp>
#include <Wt/WApplication>
#include <Wt/WEnvironment>
#include <Wt/WMenu>
#include <Wt/WMenuItem>
#include <Wt/WPopupMenu>
#include <Wt/WStackedWidget>
#include <Wt/WText>
#include <Wt/WBootstrapTheme>
#include <Wt/WTemplate>
#include <Wt/WNavigationBar>
#include <Wt/WComboBox>
#include <Wt/WVideo>
#include <Wt/WImage>
#include <Wt/WLineEdit>
/* ****************************************************************************
 * Master Menu
 * name: Name of Menu Item
 * path: Internal Path
 * type: "", "submenu", "topsub", "topend", "subsub", "subend"
 * parent: Desending from here
 * grandparent: Desending from here if sub of a sub menu
 * isSelected: True if path is selected, else false
 */
struct MasterMenu
{
    MasterMenu(const Wt::WString& name, std::string path, std::string type, std::string parent, std::string grandparent, bool isSelected) : name_(name), path_(path), type_(type), parent_(parent), grandparent_(grandparent), isSelected_(isSelected) { }
    Wt::WString name_;
    std::string path_;
    std::string type_;
    std::string parent_;
    std::string grandparent_;
    bool isSelected_;
}; // end struct MasterMenu
/* ****************************************************************************
 * DeferredWidget
 * A utility container widget which defers creation of its single
 * child widget until the container is loaded (which is done on-demand by a WMenu).
 * The constructor takes the create function for the widget as a parameter.
 *
 * We use this to defer widget creation until needed.
 */
template <typename Function>
class DeferredWidget : public Wt::WContainerWidget
{
    public:
        DeferredWidget(Function f) : f_(f) { }
    private:
        void load()
        {
            Wt::WContainerWidget::load();
            if (count() == 0) { addWidget(f_()); }
        }
        Function f_;
};
/* ****************************************************************************
 * deferCreate
 */
template <typename Function>
DeferredWidget<Function> *deferCreate(Function f)
{
    return new DeferredWidget<Function>(f);
}
/* ****************************************************************************
 * Theme
 */
struct Theme
{
    Theme(const Wt::WString& name) : name_(name) { }
    Wt::WString name_;
}; // end struct Theme
/* ****************************************************************************
 * App
 */
class App : public Wt::WApplication
{
    public:
        App(const Wt::WEnvironment& e);
    private:
        Wt::WWidget* homePage_;
        Wt::WTemplate* homeTemplate = NULL;
        Wt::WWidget* Home();
        Wt::WWidget* About();
        void CreateMenu();
        void logInternalPath(const std::string& path);
        Wt::WWidget* MenuMan();
        // Theme Vector Array
        std::vector<Theme> themes;
        // Add Theme
        void AddTheme(const Theme& l) { themes.push_back(l); }
        // Menu Vector Array
        std::vector<MasterMenu> mastermenus;
        // Add Master Menu
        void AddMenu(const MasterMenu& l) { mastermenus.push_back(l); }
        //
        void HandleThemePopup(int data);
        void SetWizardTheme(bool fromCookie, int index);
        std::string GetCookie(std::string name);
        bool SetCookie(std::string name, std::string myValue);
};
/* ****************************************************************************
 * App Contructor
 */
App::App(const Wt::WEnvironment& e) : Wt::WApplication(e)
{
    messageResourceBundle().use(appRoot() + "Theme", false);
    //
    Wt::WBootstrapTheme *bootstrapTheme = new Wt::WBootstrapTheme();
    bootstrapTheme->setVersion(Wt::WBootstrapTheme::Version3);
    setTheme(bootstrapTheme);
    //
    useStyleSheet(Wt::WApplication::resourcesUrl() + "themes/wittywizard/blue/SfMenuHoriz.css"); // FIXME: Theme
    //
    internalPathChanged().connect(this, &App::logInternalPath);

    // Add Theme
    AddTheme(Theme("Red"));
    AddTheme(Theme("White"));
    AddTheme(Theme("Blue"));
    // Set Theme
    SetWizardTheme(true, 0);
    //
    std::string myLocale = e.locale().name().c_str();
    if (myLocale == "en-US")
    {
        // Add Content to sub-menu.xml with path as id
        //                 name           path                        type       parent             grandparent, isSelected
        AddMenu(MasterMenu("home"       , "/home"                   , ""       , ""               , ""         , false)); // myMainMenu
        AddMenu(MasterMenu("Features"   , "/features"               , "topsub" , "/features"      , ""         , false)); // myMainMenu
        AddMenu(MasterMenu("Themes"     , "/features/theme"         , "submenu", "/features"      , ""         , false)); // myPopupMenu[l.path_], myMenuItem[l.path_]
        AddMenu(MasterMenu("Menu"       , "/features/menu"          , "subsub" , "/features/menu" , "/features", false)); // myMenuItem[l.path_]
        AddMenu(MasterMenu("Sub Menu"   , "/features/menu"          , "subsub" , "/features/menu" , "/features", false)); // myMenuItem[l.path_]
        AddMenu(MasterMenu("Sub Menu 1" , "/features/menu/sub-menu1", "submenu", "/features/menu" , "/features", false)); // myPopupMenu[l.path_]
        AddMenu(MasterMenu("Sub Menu"   , "/features/menu"          , "subend" , "/features/menu" , "/features", false)); // myMenuItem[l.path_]
        AddMenu(MasterMenu("Menu"       , "/features/menu"          , "subend" , "/features/menu" , "/features", false)); // myMenuItem[l.path_]
        AddMenu(MasterMenu("Features"   , "/features"               , "topend" , "/features"      , ""         , false)); // myMainMenu
        AddMenu(MasterMenu("About"      , "/about"                  , ""       , ""               , ""         , false)); // myMainMenu->addItem(myPopupMenu[l.path_]->setSubMenu(myMenuItem[l.path_]));
    }
    //else if (myLocale == "cn") { ... }
    //create_menu();
    CreateMenu();
}
/* ****************************************************************************
 * Create Menu
 */
void App::CreateMenu()
{
    bool showDebug = true;
    std::string myPath = internalPath();

    if (showDebug) { Wt::log("start") << "CreateMenu : thePath = ->" << myPath << "<-"; }
    //
    // This is our template
    homeTemplate = new Wt::WTemplate(Wt::WString::tr("ww-template"), root()); //  <message id="template">
    // Create a Container
    Wt::WContainerWidget* container = new Wt::WContainerWidget();
    // Create the stack where the contents will be located
    Wt::WStackedWidget* contents = new Wt::WStackedWidget();
    contents->setStyleClass("content");
    // FIXME
    // "|chat|blog|video|"
    //theIncludes = CrystalBall::ModuleIncludes[domainName];
    //if (showDebug) { Wt::log("notice") << "WittyWizard::CreateHome:  theIncludes=" << theIncludes; }
    // create a top-level menu
    // Setup a Left-aligned menu.
    Wt::WMenu *mainMenu_ = new Wt::WMenu(contents, container);
    mainMenu_->setStyleClass("sfhmenu");
    //mainMenu_->removeStyleClass("nav", true); // top level li of sfhmenu with children

    // ------------------------------------------------------------------------
    // query menu: lang,
    // FIXME: I do not like hard coding directories
    // FIXME: how do I know domainName is Clean
    //MenuManView* thisMenu = new MenuManView(appRoot() + "home/" + domainName + "/menuman/", *dbConnection, myLanguage, CrystalBall::UseDb[domainName], domainName, Wt::Horizontal, internalPath());
    //thisMenu->GetMenus();

    // FIXME: This Menu system is causeing the Theme not to work, click on Options -> Theme, change, then refresh page, if it remembers the theme, then it works
#define MENU
#ifdef MENU
    std::vector<std::string> parts;
    boost::split(parts, myPath, boost::is_any_of("/"));
    if (myPath == "/admin/menuman/updatexml")
    {
        if (showDebug) { Wt::log("notice") << "WittyWizard::CreateHome:  Admin Mode"; }
    }
    // This tracks menu items
    std::map <std::string, int> menuItemCounter;
    std::map <std::string, Wt::WSubMenuItem *> myPopupMenu;
    std::map <std::string, Wt::WMenu *> myMenuItem;
    std::string selectedMenuItem = "";
    //
    for (unsigned i = 0; i < parts.size(); i++)
    {
        if (showDebug) { Wt::log("notice") << "WittyWizard::CreateHome: parts size = " << parts.size() << " | parts[" << i << "]=" << parts[i]; }
    }
    // Clear Selected
    for (unsigned i = 0; i < mastermenus.size(); i++)
    {
        mastermenus.at(i).isSelected_ = false;
    }
    // Set Selected Menu Item
    for (unsigned i = 0; i < mastermenus.size(); i++)
    {
        MasterMenu& menu = mastermenus[i];
        if (menu.type_ == "submenu" || menu.type_ == "")
        {
            if (showDebug) { Wt::log("notice") << "WittyWizard::CreateHome: submenu: thePath = " << myPath << " ==  menu.path_ = " << menu.path_; }
            if (myPath == menu.path_)
            {
                if (showDebug) { Wt::log("notice") << "WittyWizard::CreateHome: isSelected_ submenu = " << menu.path_; }
                selectedMenuItem = menu.path_;
                mastermenus.at(i).isSelected_ = true;
                break;
            }
        }
    }
    // Set all Popups Selected in path
    if (!selectedMenuItem.empty())
    {
        for (unsigned i = 0; i < mastermenus.size(); i++)
        {
            MasterMenu& menu = mastermenus[i];
            if (menu.type_ == "")
            {
                if (std::find(parts.begin(), parts.end(), menu.path_) != parts.end())
                {
                    if (showDebug) { Wt::log("notice") << "WittyWizard::CreateHome: isSelected_ = " << menu.name_; }
                    mastermenus.at(i).isSelected_ = true;
                }
            }
            if (menu.type_ == "topsub")
            {
                if (std::find(parts.begin(), parts.end(), menu.path_) != parts.end())
                {
                    if (showDebug) { Wt::log("notice") << "WittyWizard::CreateHome: topsub isSelected_ = " << menu.name_; }
                    mastermenus.at(i).isSelected_ = true;
                }
            }
            if (menu.type_ == "subsub")
            {
                if (std::find(parts.begin(), parts.end(), menu.path_) != parts.end())
                {
                    if (showDebug) { Wt::log("notice") << "WittyWizard::CreateHome: subsub isSelected_ = " << menu.name_; }
                    mastermenus.at(i).isSelected_ = true;
                }
            }
            if (menu.type_ == "submenu")
            {
                if (myPath == menu.path_)
                {
                    if (std::find(parts.begin(), parts.end(), menu.path_) != parts.end())
                    {
                        if (showDebug) { Wt::log("notice") << "WittyWizard::CreateHome: submenu isSelected_ = " << menu.name_; }
                        mastermenus.at(i).isSelected_ = true;
                    }
                    break;
                }
            }
        }
    }
    // Now lets Build the Menu
    for (unsigned i = 0; i < mastermenus.size(); i++)
    {
        MasterMenu& menu = mastermenus[i];
        if (menu.type_ == "")
        {
            // Add to Main Menu
            // Note: I could not move this section into MenuManView due to boost bind done here
            mainMenu_->addItem(menu.name_, deferCreate(boost::bind(&App::MenuMan, this)))->setLink(Wt::WLink(Wt::WLink::InternalPath, "/" + menu.path_));
            if (menu.isSelected_)
            {
                if (showDebug) { Wt::log("notice") << "WittyWizard::CreateHome: isSelected_ myMainMenu->addItem(" << menu.name_; }
                mainMenu_->itemAt(menuItemCounter["rootmenu"]++)->setStyleClass("selectedmm"); // top level li of sfhmenu selected
            }
            else
            {
                if (showDebug) { Wt::log("notice") << "WittyWizard::CreateHome: myMainMenu->addItem(" << menu.name_; }
                mainMenu_->itemAt(menuItemCounter["rootmenu"]++)->setStyleClass("mainmenumm"); // top level li of sfhmenu
            } // end if (menu.isSelected_)
        }
        else
        {
            if (menu.type_ == "topsub")
            {
                if(myPopupMenu.find(menu.path_) == myPopupMenu.end())
                {
                    myPopupMenu[menu.path_] = new Wt::WSubMenuItem(menu.name_); // , "/" + l.path_ , container
                    //myPopupMenu[menu.path_]->removeStyleClass("dropdown", true); // top level li of sfhmenu with children
                    if (menu.isSelected_)
                    {
                        if (showDebug) { Wt::log("notice") << "WittyWizard::CreateHome: isSelected_ topsub myPopupMenu[" << menu.path_ << "] |  new WSubMenuItem(" << menu.name_; }
                        myPopupMenu[menu.path_]->setStyleClass("activemmwc"); // top level li of sfhmenu with children selected
                    }
                    else
                    {
                        if (showDebug) { Wt::log("notice") << "WittyWizard::CreateHome: topsub myPopupMenu[" << menu.path_ << "] |  new WSubMenuItem(" << menu.name_; }
                        myPopupMenu[menu.path_]->setStyleClass("mmwc"); // top level li of sfhmenu with children
                    } // end if (menu.isSelected_)
                    menuItemCounter["rootmenu"]++;
                } // end if(myPopupMenu.find(menu.path_) == myPopupMenu.end())
                if(myMenuItem.find(menu.path_) == myMenuItem.end())
                {
                    if (showDebug) { Wt::log("notice") << "WittyWizard::CreateHome: topsub new WMenu myMenuItem[" << menu.path_; }
                    myMenuItem[menu.path_] = new Wt::WMenu(contents, container);
                    myMenuItem[menu.path_]->setStyleClass("sfhmenu"); // top level ul under sfhmenu, container
                } // end if(myMenuItem.find(menu.path_) == myMenuItem.end())
            } // end if (l.type_ == "topsub")
            if (menu.type_ == "subsub")
            {
                if(myPopupMenu.find(menu.parent_) == myPopupMenu.end())
                {
                    myPopupMenu[menu.parent_] = new Wt::WSubMenuItem(menu.name_); // , "/" + l.path_ , container
                    if (menu.isSelected_)
                    {
                        if (showDebug) { Wt::log("notice") << "WittyWizard::CreateHome: isSelected_ subsub myPopupMenu[" << menu.parent_ << "] |  new WSubMenuItem(" << menu.name_; }
                        myPopupMenu[menu.parent_]->setStyleClass("selectedsm"); // sub level li of sfhmenu selected
                    }
                    else
                    {
                        if (showDebug) { Wt::log("notice") << "WittyWizard::CreateHome: subsub myPopupMenu[" << menu.parent_ << "] |  new WSubMenuItem(" << menu.name_; }
                        myPopupMenu[menu.parent_]->setStyleClass("smwc"); // sub level li of sfhmenu
                    } // end if (menu.isSelected_)
                } // end if(myPopupMenu.find(menu.parent_) == myPopupMenu.end())
                if(myMenuItem.find(menu.path_) == myMenuItem.end())
                {
                    if (showDebug) { Wt::log("notice") << "WittyWizard::CreateHome: subsub new WMenu myMenuItem[" << menu.parent_; }
                    myMenuItem[menu.parent_] = new Wt::WMenu(contents, container);
                    myMenuItem[menu.parent_]->setStyleClass("sfhmenu"); // sub level ul of sfhmenu
                } // end if(myMenuItem.find(menu.path_) == myMenuItem.end())
            } // end if (l.type_ == "subsub")
            if (menu.type_ == "submenu")
            {
                myMenuItem[menu.parent_]->addItem(menu.name_, deferCreate(boost::bind(&App::MenuMan, this)))->setLink(Wt::WLink(Wt::WLink::InternalPath, "/" + menu.path_));
                if (menu.isSelected_)
                {
                    if (showDebug) { Wt::log("notice") << "WittyWizard::CreateHome: isSelected_ submenu myMenuItem[" << menu.parent_ << "]->addItem(" << menu.name_; }
                    myMenuItem[menu.parent_]->itemAt(menuItemCounter[menu.parent_]++)->setStyleClass("selectedsm"); // li of sfhmenu
                }
                else
                {
                    if (showDebug) { Wt::log("notice") << "WittyWizard::CreateHome: submenu myMenuItem[" << menu.parent_ << "]->addItem(" << menu.name_; }
                    myMenuItem[menu.parent_]->itemAt(menuItemCounter[menu.parent_]++)->setStyleClass("submenumm"); // li of sfhmenu
                } // end if (menu.isSelected_)
                //myMenuItem.erase(menu.parent_);
            } // end if (l.type_ == "submenu")
            if (menu.type_ == "topend")
            {
                if (showDebug) { Wt::log("notice") << "WittyWizard::CreateHome: topend addItem myMainMenu->addItem(myPopupMenu[" << menu.parent_ << "]->setSubMenu(myMenuItem[" << menu.parent_ << "]"; }
                myPopupMenu[menu.parent_]->setSubMenu(myMenuItem[menu.parent_]);
                mainMenu_->addItem(myPopupMenu[menu.parent_]);
                //myPopupMenu.erase(menu.parent_);
            } // end if (l.type_ == "topend")
        } // end if (l.type_ != "")
        if (menu.type_ == "subend")
        {
            if (showDebug) { Wt::log("notice") << "WittyWizard::CreateHome: subend myMainMenu->addItem(myMenuItem[" << menu.grandparent_ << "]->addItem(myPopupMenu[" << menu.parent_ << "])"; }
            myPopupMenu[menu.parent_]->setSubMenu(myMenuItem[menu.parent_]);
            myMenuItem[menu.grandparent_]->addItem(myPopupMenu[menu.parent_]);
            mainMenu_->addItem(myPopupMenu[menu.grandparent_]);
        } // end if (l.type_ == "subend")
    } // end for (unsigned i = 0; i < mastermenus.size(); i++)
#else
    mainMenu_ = new Wt::WMenu(contents, container);
    mainMenu_->setStyleClass("sfhmenu");
    //mainMenu_->removeStyleClass("nav", true); // top level li of sfhmenu with children
#endif
    // ------------------------------------------------------------------------
    // Call Menu Plugin
    //CallMenuPlugin();
    /*
    // FIXME: make search work
    Wt::WText *searchResult = new Wt::WText(Wt::WString::tr("search"));
    //
    myMainMenu->addItem("Search", searchResult);
    // Add a Search control.
    Wt::WLineEdit *searchText = new Wt::WLineEdit();
    searchText->setEmptyText(Wt::WString::tr("search"));
    searchText->enterPressed().connect(std::bind([=] ()
    {
        // FIXME add a real search feature
        mainMenu_->select(4); // is the index a random menu item
        searchResult->setText(Wt::WString("Nothing found for {1}.").arg(searchText->text()));
    }));
    myMainMenu->addSearch(searchText, Wt::AlignRight);
    */

    // Setup a Right-aligned menu.
    Wt::WMenu* rightMenu = new Wt::WMenu(contents, container);
    //rightMenu->setId("rightmenu");
    rightMenu->setStyleClass("popright"); // ul
    /*
    // Create a Language popup submenu for the Language Menu.
    Wt::WSubMenuItem* languagePopup = new Wt::WSubMenuItem(Wt::WString::tr("language"));
    languagePopup->setStyleClass("popright");
    Wt::WMenu* langItems = new Wt::WMenu(contents, container);
    langItems->setStyleClass("poprightsub"); // ul
    for (unsigned i = 0; i < languages.size(); ++i)
    {
        // Get Language
        const Lang& l = languages[i];
        // Add Popup Item with Description.
        Wt::WMenuItem* mi = langItems->addItem(Wt::WString::fromUTF8(l.longDescription_.toUTF8()));
        //langItems->itemAt(i)->setStyleClass("poprightsub");
        mi->triggered().connect(boost::bind(&WittyWizard::HandleLanguagePopup, this, i));
        //if (showDebug) { Wt::log("info") << " <<< WittyWizard::CreateHome() set Language -> " << Wt::WString::fromUTF8(l.longDescription_.toUTF8()); }
    }
    // Language Popdown
    // Add Language Popup to Menu
    languagePopup->setMenu(langItems);
    rightMenu->addItem(languagePopup);
    */
    // Create a Theme popup submenu for the Theme Menu.
    Wt::WSubMenuItem* themePopup = new Wt::WSubMenuItem(Wt::WString::tr("theme"));
    themePopup->setStyleClass("popright");
    Wt::WMenu* themeItems = new Wt::WMenu(contents, container);
    themeItems->setStyleClass("poprightsub");
    for (unsigned i = 0; i < themes.size(); ++i)
    {
        // Get Theme
        const Theme& t = themes[i];
        // Add Popup Item with Description.
        Wt::WMenuItem* mit = themeItems->addItem(Wt::WString::fromUTF8(t.name_.toUTF8()));
        //themeItems->itemAt(i)->setStyleClass("poprightsub");
        mit->triggered().connect(boost::bind(&App::HandleThemePopup, this, i));
        // if (showDebug) { Wt::log("notice") << " <<<<<<<<<<<<<<< WittyWizard::CreateHome() themes " << t.name_ << " >>>>>>>>>>>>>>>>>>>>> "; }
    }
    // Theme Popdown
    themePopup->setMenu(themeItems);
    rightMenu->addItem(themePopup);
    // Now add Menu to Main Menu

    Wt::WMenuItem *moit = mainMenu_->addMenu(Wt::WString::tr("options"), rightMenu);
    moit->setStyleClass("navright"); // ul

    container->addWidget(contents);

    // On Select
    //mainMenu_->itemSelected().connect(this, &WittyWizard::GoogleAnalyticsLogger);
    // Make the menu be internal-path aware.
    //mainMenu_->setInternalPathEnabled("/"); // Not sure about this, should it be /en/ ?
    // Bind to Template
    homeTemplate->bindWidget("menu", mainMenu_);
    homeTemplate->bindWidget("contents", container);
    // Banner
    /*
    std::string myBannerSource = Wt::WString::tr("banner-source").toUTF8();
    std::string myBannerAlt = Wt::WString::tr("banner-alt").toUTF8();
    Wt::WText* banner;
    if (myBannerSource.empty())
        { banner = new Wt::WText(""); }
    else
        { banner = new Wt::WText("<img src='" + myBannerSource + "' alt='" + myBannerAlt + "'>", Wt::XHTMLUnsafeText); }
    */
    // CopyRight
    //Wt::WText* copyright = new Wt::WText("<a href='" + myBaseUrl + "'>" + Wt::WString::tr("copyright") + "</a>", Wt::XHTMLUnsafeText); // FIXME add copyright page to CMS
    // Footer Menu
    //
    //std::string menuExtras = "";
    // FIXME: Make this an xml item <message id="contacts">name1|path1|name2|path2</message> replace with contact below, split
    /*
    if (theIncludes.find("|Contact|") != std::string::npos)
    {
        menuExtras = "| <a href='" + myBaseUrl + "contact'>" + Wt::WString::tr("contact").toUTF8() + "</a>";
    }
    */
    //Wt::WText* footermenu = new Wt::WText("<a href='" + myBaseUrl + "'>" + Wt::WString::tr("home") + "</a> " + menuExtras, Wt::XHTMLUnsafeText);
    // Google Analytics
    //Wt::WText* ga = new Wt::WText("<script>/*<![CDATA[*/(function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){(i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)})(window,document,'script','//www.google-analytics.com/analytics.js','ga');ga('create', '" + CrystalBall::GaAccount[domainName] + "', '" + myHost + "');ga('send', 'pageview');/* ]]> */</script>", Wt::XHTMLUnsafeText);
    //
    //homeTemplate->bindWidget("banner", banner);
    //homeTemplate->bindWidget("copyright", copyright);
    //homeTemplate->bindWidget("footermenu", footermenu);
    //homeTemplate->bindWidget("ga", ga);
    /*
    sideBarContent_ = new WContainerWidget();
    result->bindWidget("sidebar", sideBarContent_);
    */
    // Add Home Page to root
} // end CreateMenu
/* ****************************************************************************
 * Handle Theme Popup
 */
void App::HandleThemePopup(int data)
{
    Wt::log("start") << " *** App::HandleThemePopup(data: " << data << ") *** ";
    SetWizardTheme(false, data);
} // end HandleThemePopup
/* ****************************************************************************
 * Set Theme
 */
void App::SetWizardTheme(bool fromCookie, int index)
{
    Wt::log("start") << " *** App::SetWizardTheme(FromCookie: " << "data: " << index << ") *** ";
    std::string myTheme;
    if (fromCookie)
    {
        myTheme = GetCookie("theme");
        if (myTheme.empty())
        {
            myTheme = "Blue";
        }
    }
    else
    {
        myTheme = themes[index].name_.toUTF8();
    }
    if (!myTheme.empty())
    {
        // FIXME check for legal path - Note: even if they change the cookie, its still only going to change the file name, worse case it does not work
        this->doJavaScript("document.getElementById('wittywizardstylesheet').href='" + Wt::WApplication::resourcesUrl() + "Theme-" + myTheme + ".css';");
        //useStyleSheet(Wt::WApplication::resourcesUrl() + "Theme-" + myTheme + ".css");
        SetCookie("theme", myTheme);
    } // end if (!myTheme.empty())
} // end SetWizardTheme
/* ****************************************************************************
 * Set Cookie
 */
bool App::SetCookie(std::string name, std::string myValue)
{
    Wt::WApplication *app = Wt::WApplication::instance();
    try
    {
        app->setCookie(name, myValue, 150000, "", "/", false);
    }
    catch (std::exception& e)
    {
        std::cerr << e.what() << std::endl;
        std::cerr << "App::SetCookie: Failed writting cookie: " << name;
        Wt::log("error") << "App::SetCookie()  Failed writting cookie: " << name;
        return false;
    }
    Wt::log("status") << "App::SetCookie()  Set Cookie: " << name;
    return true;
} // end SetCookie
/* ****************************************************************************
 * Get Cookie
 * std::string myCookie = GetCookie("videoman");
 */
std::string App::GetCookie(std::string name)
{
    std::string myCookie = "";
    try
    {
        myCookie = Wt::WApplication::instance()->environment().getCookie(name);
    }
    catch (std::exception& e)
    {
        std::cerr << e.what() << std::endl;
        std::cerr << "App::GetCookie: Failed reading cookie: " << name;
        Wt::log("error") << "App::GetCookie()  Failed reading cookie: " << name;
    }
    return myCookie;
} // end std::string App::GetCookie
/* ****************************************************************************
 * Menu Man
 */
Wt::WWidget* App::MenuMan()
{
    return new Wt::WText(Wt::WString::tr("page.home")); // just a bogus test
} // end MenuMan
/* ****************************************************************************
 * Home
 */
Wt::WWidget* App::Home()
{
    return new Wt::WText(Wt::WString::tr("page.home"));
}
/* ****************************************************************************
 * About
 */
Wt::WWidget* App::About()
{
    return new Wt::WText(Wt::WString::tr("page.about"));
}
/* ****************************************************************************
 * logInternalPath
 */
void App::logInternalPath(const std::string& path)
{
    // simulate an access log for the interal paths
    log("Path Change") << path;
}
/* ****************************************************************************
 * create_app
 */
Wt::WApplication* create_app(const Wt::WEnvironment& env)
{
    return new App(env);
}
/* ****************************************************************************
 * main
 */
int main(int argc, char* argv[])
{
    return Wt::WRun(argc, argv, create_app);
}
// --- End of File ------------------------------------------------------------
